<?php
if (!defined('ABSPATH')) {
    exit;
}

class BW_Variation_Settings {
    private static $instance = null;
    private $options;

    /**
     * Gets the current instance of the settings class.
     *
     * Returns a shared instance of the settings class.
     *
     * @since 1.0.0
     *
     * @return BW_Variation_Settings The current instance.
     */
    public static function get_instance() {
        if (null === self::$instance) {
            self::$instance = new self();
        }
        return self::$instance;
    }

    /**
     * Private constructor for the settings class.
     *
     * Registers the admin menu and init actions and loads the current options from the database.
     *
     * @since 1.0.0
     */
    private function __construct() {
        add_action('admin_menu', array($this, 'add_plugin_page'));
        add_action('admin_init', array($this, 'page_init'));
        $this->options = get_option('bw_variation_meta_fields', array());
    }

    /**
     * Adds a submenu page under the WooCommerce menu for managing variation meta fields.
     *
     * This function registers a new submenu page titled "Variation Meta Fields" under
     * the WooCommerce menu in the WordPress admin. The page is accessible to users with
     * the 'manage_woocommerce' capability and is rendered by the `create_admin_page` method.
     *
     * @since 1.0.0
     */
    public function add_plugin_page() {
        add_submenu_page(
            'woocommerce',
            __('Variation Meta Fields', 'woocommerce-custom-variation-meta'),
            __('Variation Meta Fields', 'woocommerce-custom-variation-meta'),
            'manage_woocommerce',
            'bw-variation-meta-settings',
            array($this, 'create_admin_page')
        );
    }

    /**
     * Renders the admin page for managing variation meta fields.
     *
     * This function outputs the HTML for the admin settings page that allows
     * users to manage custom variation meta fields. It includes a form for
     * submitting the settings to the WordPress options API, as well as a
     * template for adding new meta fields dynamically using JavaScript.
     */
   public function create_admin_page() {
    ?>
        <div class="wrap">
            <h2><?php _e('Variation Meta Fields Settings', 'woocommerce-custom-variation-meta'); ?></h2>
            <form method="post" action="options.php">
                <?php
                settings_fields('bw_variation_meta_group');
                do_settings_sections('bw-variation-meta-settings');
                submit_button();
                ?>
            </form>
            <div id="field-template" style="display:none">
                <div class="meta-field">
                    <input type="text" name="bw_variation_meta_fields[labels][]" value="" 
                        placeholder="<?php _e('Field Label', 'woocommerce-custom-variation-meta'); ?>">
                    <input type="text" name="bw_variation_meta_fields[descriptions][]" value="" 
                        placeholder="<?php _e('Field Description', 'woocommerce-custom-variation-meta'); ?>">
                    <button type="button" class="button remove-field"><?php _e('Remove', 'woocommerce-custom-variation-meta'); ?></button>
                </div>
            </div>
        </div>
        <?php
    }


    /**
     * Initializes the settings page for variation meta fields.
     *
     * This function registers the settings, adds a settings section, and defines
     * the settings fields for the variation meta fields admin page. It hooks into
     * the WordPress settings API to enable the saving and sanitization of the meta
     * fields options.
     *
     * @since 1.0.0
     */
    public function page_init() {
        register_setting(
            'bw_variation_meta_group',
            'bw_variation_meta_fields',
            array($this, 'sanitize')
        );

        add_settings_section(
            'bw_variation_meta_section',
            __('Custom Meta Fields', 'woocommerce-custom-variation-meta'),
            array($this, 'section_info'),
            'bw-variation-meta-settings'
        );

        add_settings_field(
            'meta_fields',
            __('Meta Fields', 'woocommerce-custom-variation-meta'),
            array($this, 'meta_fields_callback'),
            'bw-variation-meta-settings',
            'bw_variation_meta_section'
        );
    }

    /**
     * Prints the section info text on the admin settings page.
     *
     * This function is a callback for the 'bw_variation_meta_section' settings section, and is responsible for
     * printing the section title and description on the admin settings page.
     *
     * @since 1.0.0
     */
    public function section_info() {
        echo __('Add your custom variation meta fields below:', 'woocommerce-custom-variation-meta');
    }

    /**
     * Renders the form fields for adding and editing custom variation meta fields on the admin settings page.
     *
     * This function is a callback for the 'meta_fields' settings field, and is responsible for rendering
     * the form fields for adding and editing custom variation meta fields. It uses the currently saved
     * options to populate the form fields, and includes a template for adding new fields dynamically
     * using JavaScript.
     */
    public function meta_fields_callback() {
        $fields = isset($this->options['fields']) ? $this->options['fields'] : array();
        $labels = isset($this->options['labels']) ? $this->options['labels'] : array();
        $descriptions = isset($this->options['descriptions']) ? $this->options['descriptions'] : array();
        $hide_fields = isset($this->options['hide']) ? $this->options['hide'] : array();
        ?>
        <div id="meta-fields-container">
            <?php
            if (!empty($labels)) {
                foreach ($labels as $key => $label) {
                    ?>
                    <div class="meta-field">
                        <input type="text" name="bw_variation_meta_fields[labels][]" 
                            value="<?php echo esc_attr($label); ?>" 
                            placeholder="<?php _e('Field Label', 'woocommerce-custom-variation-meta'); ?>">
                        <input type="text" name="bw_variation_meta_fields[descriptions][]" 
                            value="<?php echo esc_attr($descriptions[$key]); ?>" 
                            placeholder="<?php _e('Field Description', 'woocommerce-custom-variation-meta'); ?>">
                        <label class="hide-field-label">
                            <input type="checkbox" name="bw_variation_meta_fields[hide][]" 
                                value="<?php echo esc_attr($fields[$key]); ?>" 
                                <?php checked(in_array($fields[$key], $hide_fields)); ?>>
                            <?php _e('Hide on product', 'woocommerce-custom-variation-meta'); ?>
                        </label>
                        <button type="button" class="button remove-field"><?php _e('Remove', 'woocommerce-custom-variation-meta'); ?></button>
                    </div>
                    <?php
                }
            }
            ?>
        </div>
        <button type="button" class="button" id="add-field"><?php _e('Add Field', 'woocommerce-custom-variation-meta'); ?></button>
        <?php
    }


    /**
     * Sanitizes the user input before saving it to the database.
     *
     * This function is a callback for the 'sanitize_callback' argument of the 'register_setting' function.
     * It takes the user input as an array, and returns a sanitized version of it.
     *
     * @since 1.0.0
     *
     * @param array $input The user input to be sanitized.
     * @return array The sanitized user input.
     */
    public function sanitize($input) {
        $new_input = array();
        
        if(isset($input['labels'])) {
            $new_input['labels'] = array_map('sanitize_text_field', $input['labels']);
            
            // Générer les fields à partir des labels
            $new_input['fields'] = array_map(function($label) {
                return sanitize_title($label); // Convertit le label en slug
            }, $input['labels']);
        }
        
        if(isset($input['descriptions'])) {
            $new_input['descriptions'] = array_map('sanitize_text_field', $input['descriptions']);
        }
        
        if(isset($input['hide'])) {
            $new_input['hide'] = array_map('sanitize_text_field', $input['hide']);
        }
        
        return $new_input;
    }


    /**
     * Retrieves the custom variation meta fields as an array.
     *
     * The returned array will have the following keys:
     * - fields: an array of custom field IDs
     * - labels: an array of custom field labels
     * - descriptions: an array of custom field descriptions
     * - hide: an array of custom field IDs that should be hidden on the frontend
     *
     * @since 1.0.0
     *
     * @return array The custom variation meta fields array.
     */
    public function get_meta_fields() {
        return $this->options;
    }
}